/*
 *  @file   FrameQ.c
 *
 *  @brief  Front end  implementation of FrameQ module on RTOS side(DSP/BIOS).
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */



/*-------------------------    XDC  specific includes ----------------------  */
#include <xdc/std.h>
#include <xdc/runtime/Memory.h>
#include <xdc/runtime/System.h>
#include <xdc/runtime/Startup.h>
#include <xdc/runtime/Assert.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/IGateProvider.h>
#include <xdc/runtime/Gate.h>



/*-------------------------  BIOS includes ---------------------------------- */
#include <ti/sysbios/gates/GateMutex.h>

#ifdef xdc_target__isaCompatible_64P
#include <ti/sysbios/family/c64p/Cache.h>
#else
#include <ti/sysbios/hal/Cache.h>
#endif
/*-------------------------    Generic includes  ---------------------------- */
#include <string.h>

/*-------------------------    IPC module specific includes ------------------*/
#include <ti/sdo/ipc/Ipc.h>




#include "package/internal/FrameQ.xdc.h"

#include <ti/syslink/ipc/rtos/interfaces/_SysLink.h>

#include <ti/syslink/ipc/rtos/interfaces/IFrameQ.h>
#include <ti/sdo/utils/_NameServer.h>
#include <ti/sdo/utils/_MultiProc.h>
#include <ti/sdo/ipc/_SharedRegion.h>
#include <ti/sdo/ipc/_ListMP.h>

#include <ti/syslink/ipc/rtos/_FrameQBufMgr.h>
#include <ti/syslink/ipc/rtos/shMem/_FrameQ_ShMem.h>
#include <ti/syslink/ipc/rtos/_FrameQ.h>

/* Reserved name used for name server instance of the FrameQ.
 * Note: Applications should not used this name for the instance creation
 */
#define FRAMEQ_NAMESERVERNAME  "FrameQ"

Ptr gFrameQ_nsHandle; /* TODO: remove this.Hack for NameServer_getHandle issue in nameserver */
/* Internal API declarations */
Int32
_FrameQ_create (FrameQ_Object  *obj,
                const ti_syslink_ipc_rtos_FrameQ_Params  *params,
                Error_Block    *eb);

Int32
 _FrameQ_open (FrameQ_Object* obj,const ti_syslink_ipc_rtos_FrameQ_Params* params,
               Error_Block *eb);

/** ============================================================================
 *  External APIs
 *  ============================================================================
 */

/*
 *  ======== FrameQ_getConfig ========
 *  Function to get the default configuration for the FrameQ module.
 *
 *  This function can be called by the application to get their
 *  configuration parameter to FrameQ_setup filled in by the
 *  FrameQ module with the default parameters. If the user does
 *  not wish to make any change in the default parameters, this API
 *  is not required to be called.
 *
 */
Int32
FrameQ_getConfig (FrameQ_Config *cfg)
{
    Int32  status = FrameQ_S_SUCCESS;

    if (NULL != cfg) {
        /* (If setup has not yet been called) */
        if (FrameQ_module->refCount == 0) {
            /* Return the default static config values */
            cfg->eventNo = FrameQ_NOTIFY_RESERVED_EVENTNO;
            cfg->maxInstances  = FrameQ_MAX_INSTANCES;
        }
        else {
            /* Return the currently configure parameters */
            memcpy (cfg,
                    &FrameQ_module->cfg,
                    sizeof (FrameQ_Config));
        }
    }
    else {
        status = FrameQ_E_INVALIDARG;
    }

    return (status);
}


/*
 *  ======== FrameQ_sharedMemReq ========
 *  API to  return the shared memory required for the instance based on the type
 * of implementation.
 */
UInt32
FrameQ_sharedMemReq (Ptr params)
{
    UInt32                 totalSize     = 0;
    FrameQ_CreateParams    *frmQCreParams;

    if (NULL != params) {

        frmQCreParams = (FrameQ_CreateParams *)params;

        switch (frmQCreParams->ctrlInterfaceType) {
            case FrameQ_INTERFACE_SHAREDMEM:
                {
                    /* Call shared memory implementation's API*/
                    totalSize = FrameQ_ShMem_sharedMemReq(params);
                }
                break;

            default:
                /* Interface type is not known.*/
                break;
        }
    }

    return (totalSize);
}

/*
 *  ======== FrameQ_create ========
 *  Function to create a FrameQ.params should be of type specific to
 *  implementation.
 *  For example if apps want to create FrameQ using SharedMemory implementation
 *  they have to pass FrameQ_ShMem_Params to this create call.
 */
FrameQ_Handle
FrameQ_create (Ptr params)
{
    Int32 status = FrameQ_S_SUCCESS;
    ti_syslink_ipc_rtos_FrameQ_Params fqParams;
    FrameQ_Params   *appfqParams = (FrameQ_Params *)params;
    FrameQ_Handle fqHandle =  NULL;
    ti_syslink_ipc_rtos_shMem_FrameQ_ShMem_Params shMemParams;

    Error_Block eb;

    Error_init(&eb);
    ti_syslink_ipc_rtos_FrameQ_Params_init(&fqParams);

    memcpy(&fqParams.commonCreateParams,
           (Ptr)&(appfqParams->commonCreateParams),
           sizeof (ti_syslink_ipc_rtos_FrameQ_CreateParams));

    /* Initialize the implementation specific params and pass it as impParams*/
    switch (appfqParams->commonCreateParams.ctrlInterfaceType)
    {
        case FrameQ_INTERFACE_SHAREDMEM:
                ti_syslink_ipc_rtos_shMem_FrameQ_ShMem_Params_init(
                                                                  &shMemParams);
            /* Copy the application  shmem params directly starting fromthe common
             * create params
             */
            memcpy(&(shMemParams.commonCreateParams),
                   params,
                   sizeof (FrameQ_ShMem_Params));
            fqParams.impParams = &shMemParams;
            fqParams.impParamsSize =
                      sizeof(ti_syslink_ipc_rtos_shMem_FrameQ_ShMem_Params);
        break;
        default:
            status = FrameQ_E_FAIL;
    }

    if (status >= 0) {
        /* Nothing to be done for this function. */
        fqHandle = ti_syslink_ipc_rtos_FrameQ_create(&fqParams, &eb);
        if (fqHandle == NULL) {
            Error_print(&eb);
            Error_init(&eb);
        }
    }

    return (fqHandle);
}

/*
 *  ======== FrameQ_delete ========
 *  Function to delete the created FrameQ instance.
 */
Int32
FrameQ_delete (FrameQ_Handle * handle)
{
    /* Nothing to be done for this function. */
    ti_syslink_ipc_rtos_FrameQ_delete(handle);

    return(0);
}


/*
 *  ======== FrameQ_instance_init ========
 *  API to create the  instance. Application's call FrameQ_create will drop to
 *  this API though xdc  runtime.
 */
Int32
FrameQ_Instance_init (FrameQ_Object *obj,
                      const ti_syslink_ipc_rtos_FrameQ_Params *params,
                      Error_Block *eb)
{
    Int32                   status      = FrameQ_S_SUCCESS;

    if (params == NULL) {
        status = FrameQ_E_INVALIDARG;
        Error_raise(eb,
                    FrameQ_E_InvalidArg,
                    NULL,
                    0);
    }
    else if (params->commonCreateParams.openFlag == TRUE) {
       status = _FrameQ_open (obj,params, eb);
       if (status < 0) {
            status = FrameQ_E_FAIL;
            Error_raise(eb,
                        FrameQ_E_FrameQOpenFail,
                        NULL,
                        0);
       }
    }
    else if (params->commonCreateParams.openFlag == FALSE) {
        status = _FrameQ_create (obj, params, eb);
        if (status < 0) {
            status = FrameQ_E_FAIL;
        }
    }
    return (status);
}

/*
 * ======== FrameQ_Instance_finalize ========
 * Function to delete the created FrameQ instance
 */
void
FrameQ_Instance_finalize (FrameQ_Object *obj, int instStatus)
{
    FrameQ_ShMem_Handle handle;

    Assert_isTrue ((NULL != obj), FrameQ_A_invalidParams);

    switch (obj->objType) {

    case FrameQ_DYNAMIC_OPEN:
    {
        /* Close the implementation specific handle and delete the top level
         * Object.
         */
        if (obj->handle != NULL) {
            switch (obj->ctrlInterfaceType) {
                case FrameQ_INTERFACE_SHAREDMEM:
                {
                    handle = FrameQ_ShMem_Handle_downCast(obj->handle);
                    FrameQ_ShMem_close(&handle);
                }
                break;

                default:
                break;
            }
        }

    }
    break;

    case FrameQ_DYNAMIC_CREATE:
    {
        /* Delete the implementation specific handle and delete the top level
         * Object.
         */
        if (obj->handle != NULL) {
            IFrameQ_delete((IFrameQ_Handle*)&obj->handle);
        }
        break;

    }

    case FrameQ_STATIC_CREATE:
    {
        /* TODO: raise error or some assert */
        break;
    }

    case FrameQ_STATIC_OPEN:
    {
        /* TODO: raise error or some assert */
        break;
    }
    default:
            break;
    }

}

/*
 * ======== FrameQ_open ========
 * Function to open the created FrameQ instance when name is known and
 * name server is configured.The instance must be created before opening it.
 * Application is expected to pass the implementation specific open params for
 * the openParams field.
 */
Int32
FrameQ_open (Ptr  intOpenParams, FrameQ_Handle * handlePtr)
{
    Int32                   status     = FrameQ_S_SUCCESS;
    FrameQ_Handle           handle    = NULL;
    FrameQ_Object          *obj;
    FrameQ_OpenParams      *openParams =
                                       (FrameQ_OpenParams *)intOpenParams;
    Ptr                     impHandle = NULL;
    ti_syslink_ipc_rtos_FrameQ_Params    params;
    FrameQ_NameServerEntry  entry;
    FrameQ_NameServerEntry  *pentry;
    UInt32                  interfaceType ;
    UInt8                   nameLen;
    Int32                   key;
    UInt32                  index;
    UInt32                  len;
    Error_Block             eb;

    Assert_isTrue ((NULL != handlePtr), FrameQ_A_invalidParams);
    Assert_isTrue ((NULL != openParams), FrameQ_A_invalidParams);

    if ((handlePtr == NULL) ||(openParams == NULL)) {
        status = FrameQ_E_INVALIDARG;
    }
    else {
        Error_init(&eb);
        /* Get implementation type by querying  the name server and call
         * Specific implementation's open call.
         */
        nameLen = strlen((String)openParams->commonOpenParams.name);
        if (openParams->commonOpenParams.name != NULL) {
            /* If name is provided use name based  open only */
            openParams->commonOpenParams.sharedAddr = NULL;
        }

        if (   (nameLen == 0)
            && (openParams->commonOpenParams.sharedAddr == NULL)) {
            /*if  both name and shared addr is not provided return error */
            status = FrameQ_E_INVALIDARG;
        }
        else if (   (   (nameLen > 0)
                     && ( nameLen <= (FrameQ_MAX_NAMELEN-1))
                     && (NULL != FrameQ_module->nameServer))) {
            len = sizeof (FrameQ_NameServerEntry);
            status = NameServer_get ((NameServer_Handle)FrameQ_module->nameServer,
                                     openParams->commonOpenParams.name,
                                     &entry,
                                     &len,
                                     NULL);
            if (status < 0) {
                status = FrameQ_E_NOTFOUND;
            }
            else {
                interfaceType = entry.ctrlInterfaceType;
            }
        }
        else if (openParams->commonOpenParams.sharedAddr != NULL) {
            pentry = (FrameQ_NameServerEntry*)
                                       openParams->commonOpenParams.sharedAddr;
            /* Assert that sharedAddr is cache aligned */
            index = SharedRegion_getId(pentry);

            /* Assert that the buffer is in a valid shared region */
            Assert_isTrue(index != SharedRegion_INVALIDREGIONID,
                          Ipc_A_addrNotInSharedRegion);

            Assert_isTrue(((UInt32)pentry %
                          SharedRegion_getCacheLineSize(index) == 0),
                          Ipc_A_addrNotCacheAligned);

            if (SharedRegion_isCacheEnabled(index)) {
                /* Invalidating the entry located in shared memory. The memory
                 * provided by this entry should be in cache aligend size
                 */
                key = Gate_enterSystem();
                Cache_inv (pentry,
                           sizeof (FrameQ_NameServerEntry),
                           Cache_Type_ALL,
                           TRUE);
                Gate_leaveSystem (key);
            }
            interfaceType = pentry->ctrlInterfaceType;
        }
        else {
            status = FrameQ_E_INVALIDARG;
        }

        if (status >= 0) {
            /* Check the interface type and if it is not supported
             * return failure.
             */
            if (!FrameQ_isSupportedInterface(interfaceType)) {
                status = FrameQ_E_INVALID_INTERFACE;
                Error_raise(&eb,
                            FrameQ_E_InvalidInterfaceType,
                            NULL,
                            0);
            }
            else {
                ti_syslink_ipc_rtos_FrameQ_Params_init (&params);
                params.commonCreateParams.name = (String)
                                             openParams->commonOpenParams.name;
                /* Opening the instance  */
                params.commonCreateParams.openFlag = TRUE;

                handle = ti_syslink_ipc_rtos_FrameQ_create (&params, &eb);

                if (handle == NULL) {
                    /* Failed to create the top level FrameQ object */
                    status = FrameQ_E_FAIL;
                }
                else {
                    /* Open the handle to the implementation specific handle
                     * and plug in to the top level FrameQ object.
                     */
                    switch (entry.ctrlInterfaceType) {
                        case FrameQ_INTERFACE_SHAREDMEM:
                            {
                                status = FrameQ_ShMem_open(
                                         (ti_syslink_ipc_rtos_shMem_FrameQ_ShMem_OpenParams *)openParams,
                                         (FrameQ_ShMem_Handle *)&impHandle);
                                if ((status >= 0) && (impHandle != NULL)) {
                                    impHandle = FrameQ_ShMem_Handle_upCast(
                                                                     impHandle);
                                }
                                else {
                                    FrameQ_close(&handle);
                                }

                            }
                        break;

                        default:
                            /* NameServer entry does not contain the
                             * implementation type.
                             */
                            status = FrameQ_E_INVALID_INTERFACE;
                    }
                }
            }
        }

    }

    if (status >= 0) {
        obj = handle;
        obj->handle        = impHandle;
        obj->ctrlInterfaceType = interfaceType;
        obj->objType       = FrameQ_DYNAMIC_OPEN;
        *handlePtr         = handle;
    }
    else {
       *handlePtr     = NULL;
    }

    return (status);
}

/*
 * ======== FrameQ_close ========
 * Function to close the dynamically opened shared memory FrameQ instance.
 */
Int32
FrameQ_close(FrameQ_Handle* pHandle)
{
    Int32 status = FrameQ_S_SUCCESS;
    Assert_isTrue ((NULL != pHandle),
                    FrameQ_A_invalidParams);
    /*TODO: Close  only dynamically opened instance */

    FrameQ_delete(pHandle);

    return (status);
}

/*
 *  ======== FrameQ_alloc ========
 * Function to allocate a  frame.
 * It allocates frame from the free Queue 0 of FrameQBufMgr.
 */
Int32
FrameQ_alloc (FrameQ_Object      *obj,
              FrameQ_Frame       * framePtr)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != framePtr),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_alloc(obj->handle,
                          (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame*)
                           framePtr);

    return (status);
}

/*
 *  ======== FrameQ_allocv ========
 * Function to allocate multiple frames from the free frame queues given
 * (free frame queues are configured in FrameQbufMgr).
 * After API returns,numframes will denotes  the number of successfully
 * allocated frames.
 */
Int32
FrameQ_allocv (FrameQ_Object *obj,
               FrameQ_Frame  framePtr[],
               UInt32        freeQId[],
               UInt8         *numFrames)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != framePtr),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != freeQId),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFrames),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_allocv(obj->handle,
                            (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame*)framePtr,
                            freeQId,
                            numFrames);

    return (status);
}

/*
 *  ======== FrameQ_free ========
 * Function to free single frame
 */
Int32
FrameQ_free (FrameQ_Object *obj, FrameQ_Frame  frame)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != frame),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_free(obj->handle,
                          (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame)frame);

    return (status);

}

/*
 *  ======== FrameQ_freev ========
 * Function to free  multiple frames
 */
Int32
FrameQ_freev (FrameQ_Object *obj, FrameQ_Frame  framePtr[], UInt32 numFrames)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != framePtr),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFrames),
                   FrameQ_A_invalidParams);


    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_freev(obj->handle,
                          (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame*)framePtr,
                           numFrames);

    return (status);

}


/*
 * ======== FrameQ_put ========
 * Function to  insert frame in to FrameQ. It inserts frame in to filled
 * frames queue 0 of FrameQ reader client.
 */
Int32
FrameQ_put  (FrameQ_Object *obj, FrameQ_Frame  frame)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != frame),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_put(obj->handle,
                        (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame)frame);

    return (status);

}

/*
 * ======== FrameQ_putv ========
 * Function to insert multiple frames  in to queues of a
 * FrameQ reader client.
 */
Int32
FrameQ_putv (FrameQ_Object          *obj,
             FrameQ_Frame           framePtr[],
             UInt32                 filledQueueId[],
             UInt8                  numFrames)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != framePtr),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != filledQueueId),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFrames),
                   FrameQ_A_invalidParams);


    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_putv(obj->handle,
                         (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame*)framePtr,
                         filledQueueId,
                         numFrames);

    return (status);
}

/*
 * ======== FrameQ_get ========
 * Function to retrieve frame from FrameQ
 */
Int32
FrameQ_get  (FrameQ_Object *obj, FrameQ_Frame * pframe)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != pframe),
                   FrameQ_A_invalidParams);

    /* Call implementation specific get.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_get(obj->handle,
                         (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame*)pframe);

    return (status);
}

/*
 * ======== FrameQ_getv ========
 * Function to retrieve frames from  queues of a FrameQ Reader client.
 * It returns frame if it is available in the queue.
 */
Int32
FrameQ_getv (FrameQ_Object         *obj,
             FrameQ_Frame           pframe[],
             UInt32                 filledQueueId[],
             UInt8                 *numFrames)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != pframe),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != filledQueueId),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFrames),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_getv(obj->handle,
                         (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame *)pframe,
                         filledQueueId,
                         numFrames);

    return (status);
}

/*
 * ======== FrameQ_dup ========
 * Function to duplicate given frame
 */
Int32
FrameQ_dup (FrameQ_Object           *obj,
            FrameQ_Frame            frame,
            FrameQ_Frame*           dupedFrame)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != frame),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != dupedFrame),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_dup( obj->handle,
                         (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame)frame,
                         (ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame*)dupedFrame);

    return (status);
}

/*
 *  ======== FrameQ_registerNotifier ========
 * Function to register a call back function with the FrameQ.
 *
 *  Incase of writer:-
 *      Watermark is same for all free frame Queues  of FrameQbufMgr used by
 *      this instance.
 *      Single notification function.
 *      Same logic is applied for all free queues. If it is notification condition
 *      is true for any free queue, send notification.
 *      Only one notification is generated for one free call to writer.
 *  Incase of reader:-
 *      Watermark is same for all queues of a reader.
 *      Single notification function.
 *      Same logic is applied for all queues. If it is notification condition
 *      is true for any queue, send notification.
 *      Only one notification is generated for one put/putv call to reader.
 */
Int32
FrameQ_registerNotifier (FrameQ_Object          * obj,
                         FrameQ_NotifyParams * params)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != params),
                   FrameQ_A_invalidParams);

    /* Call implementation specific function.
     * Here obj->handle is the upcasted handle.
     */
    params->cbClientHandle = obj;
    status = IFrameQ_registerNotifier (obj->handle,
                 (ti_syslink_ipc_rtos_interfaces_IFrameQ_NotifyParams *)params);

    return (status);
}

/*
 *  ======== FrameQ_unregisterNotifier ========
 * Function to unregister the call back function.
 */
Int32
FrameQ_unregisterNotifier (FrameQ_Object * obj)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_unregisterNotifier (obj->handle);

    return (status);
}

/*
 * ======== FrameQ_sendNotify ========
 * Function to send forced notification to the reader clients.
 */
Int32
FrameQ_sendNotify (FrameQ_Object * obj, UInt16 msg)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_sendNotify (obj->handle, msg);

    return (status);
}

/*
 *  ======== FrameQ_getNumFrames ========
 * For Writer: This function is to get the available frames in filled queue0 of
 *             primary quque for writer.
 * For Reader: This function is to get the available frames in filled queue0 of
 *             the caller client.
 */
Int32
FrameQ_getNumFrames (FrameQ_Object *obj, UInt32* numFrames)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFrames),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_getNumFrames (obj->handle, numFrames);

    return (status);
}

/*
 *  ======== FrameQ_getvNumFrames ========
 * For Reader this function is to get the available frames  from the queues
 * associated with this reader client.
 * For writer this function is to get the available frames  from the queues
 * associated with the primary Reader(first reader).
 */
Int32
FrameQ_getvNumFrames (FrameQ_Object   * obj,
                       UInt32           numFrames[],
                       UInt8            filledQId[],
                       UInt8            numFilledQids)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFrames),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != filledQId),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((0 < numFilledQids),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_getvNumFrames(obj->handle,
                                   numFrames,
                                   filledQId,
                                   numFilledQids);
    return (status);
}

/*
 *  ======== FrameQ_getNumFreeFrames ========
 *  Function to get the number of free frames available in the plugged in
 *  FrameQbufMgr's FreeFramePool 0. The returned number free frame may not
 *  exist in the frameQbufMgr  after this call.This is because
 *  if any allocs are done by the other FrameQ writers on the
 *  same FrameQBufMgr.If frees are not other frameQ writers the free Frames
 *  would be even more than the value that we have got.
 *
 */
Int32
FrameQ_getNumFreeFrames (FrameQ_Object *obj, UInt32* numFreeFrames)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFreeFrames),
                   FrameQ_A_invalidParams);

    /* Call implementation specific API.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_getNumFreeFrames (obj->handle, numFreeFrames);

    return (status);
}

/*
 *  ======== FrameQ_getvNumFreeFrames ========
 * Function to get the number of free frames available in the plugged in
 * FrameQbufMgr.
 */
Int32
FrameQ_getvNumFreeFrames (FrameQ_Object   * obj,
                          UInt32            numFreeFrames[],
                          UInt8             freeQId[],
                          UInt8             numFreeQids)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != numFreeFrames),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((NULL != freeQId),
                   FrameQ_A_invalidParams);

    Assert_isTrue ((0 != numFreeQids),
                   FrameQ_A_invalidParams);

    /* Call implementation specific API.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_getvNumFreeFrames(obj->handle,
                                       numFreeFrames,
                                       freeQId,
                                       numFreeQids);
    return (status);
}

/*
 *  ======== FrameQ_control ========
 *  @brief  Provides a hook to perform implementation dependent operation.
 *
 */
Int32
FrameQ_control (FrameQ_Object *obj,
                Int32          cmd,
                Ptr            arg)
{
    Int32   status = FrameQ_S_SUCCESS;

    Assert_isTrue ((NULL != obj),
                   FrameQ_A_invalidParams);

    /* Call implementation specific alloc.
     * Here obj->handle is the upcasted handle.
     */
    status = IFrameQ_control(obj->handle,
                             cmd,
                             arg);
    return (status);
}

/*=============================================================================
 * Helper APIs
 *=============================================================================
 */

/*
 *  ======== FrameQ_getExtendedHeaderPtr ========
 * API  to get the pointer to the extended header.Apps has to use this API to
 * get  pointer  to their application defined exteneded header that starts after
 * the bsee frame header.
 */
Ptr FrameQ_getExtendedHeaderPtr(FrameQ_Frame frame)
{
    Ptr    extHeaderPtr = NULL;
    UInt32 offset;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);

    offset = (  sizeof (FrameQ_FrameHeader)
              + (   (frame->numFrameBuffers - 1)
                  * sizeof (FrameQ_ShMem_FrameBufInfo)));

    extHeaderPtr = (Ptr) ( (UInt32)frame + offset);

    return(extHeaderPtr);
}

/*
 *  ======== FrameQ_ShMem_getNumFrameBuffers ========
 *  API to find out the number of frame buffers in a frame.
 */
UInt32
FrameQ_getNumFrameBuffers (FrameQ_Frame frame )
{
    UInt32 numbufs = 0;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);

    numbufs =  frame->numFrameBuffers;

    return (numbufs);
}

/*
 *  ======== FrameQ_getFrameBuffer ========
 *  API to retrieve framebuffer  identified by frameBufNum from a given frame.
 * Here no validation is performed for frameBufNum. User is expected to pass the
 * right value for it.
 * ( APPs can use getNumFrameBuffers API to know the number of frame buffers
 *  associated with the frame)
 */
Ptr
FrameQ_getFrameBuffer( FrameQ_Frame frame, UInt32 frameBufNum )
{
    Ptr                  frmBuffer = NULL;
    FrameQ_FrameBufInfo *frameBufInfo;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);
    Assert_isTrue ((frame->numFrameBuffers >= frameBufNum),
                    FrameQ_A_invalidParams);

    frameBufInfo = (FrameQ_FrameBufInfo *)&(frame->frameBufInfo[0]);

    frmBuffer = (Ptr) frameBufInfo[frameBufNum].bufPtr;

    return (frmBuffer);
}

/*
 *  ======== FrameQ_getFrameBufSize ========
 *  API to get the size of the framebuffer.
 */
UInt32
FrameQ_getFrameBufSize(FrameQ_Frame frame, UInt32 frameBufNum )
{
    UInt32               size;
    FrameQ_FrameBufInfo *frameBufInfo;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);
    Assert_isTrue ((frame->numFrameBuffers >= frameBufNum),
                    FrameQ_A_invalidParams);

    frameBufInfo = (FrameQ_FrameBufInfo *)&(frame->frameBufInfo[0]);
    size = frameBufInfo[frameBufNum].bufSize;

    return (size);
}


/*
 *  ======== FrameQ_getFrameBufValidSize ========
 *  API to get the valid data size of a framebuffer  identified by frameBufNum
 *  in a given frame.
 */
UInt32
FrameQ_getFrameBufValidSize(FrameQ_Frame frame, UInt32 frameBufNum )
{
    UInt32               size;
    FrameQ_FrameBufInfo *frameBufInfo;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);
    Assert_isTrue ((frame->numFrameBuffers >= frameBufNum),
                    FrameQ_A_invalidParams);

    frameBufInfo = (FrameQ_FrameBufInfo *)&(frame->frameBufInfo[0]);
    size = frameBufInfo[frameBufNum].validSize;

    return (size);
}

/*
 *  ======== FrameQ_getFrameBufDataStartOffset ========
 *  API to get the valid data start offset in framebuffer  identified by
 *  frameBufNum in a given frame.
 */
UInt32
FrameQ_getFrameBufDataStartOffset(FrameQ_Frame frame,
                                        UInt32         frameBufNum )
{
    UInt32               offSet;
    FrameQ_FrameBufInfo *frameBufInfo;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);
    Assert_isTrue ((frame->numFrameBuffers >= frameBufNum),
                    FrameQ_A_invalidParams);

    frameBufInfo = (FrameQ_FrameBufInfo *)&(frame->frameBufInfo[0]);
    offSet = frameBufInfo[frameBufNum].startOffset;

    return (offSet);
}

/*
 *  ======== FrameQ_setFrameBufValidSize ========
 *  API to set the valid data size of a framebuffer  identified by
 *  frameBufNum in  given frame.
 */
Int32
FrameQ_setFrameBufValidSize(FrameQ_Frame frame,
                            UInt32       frameBufNum,
                            UInt32       validDataSize )
{
    Int32                      status       = FrameQ_S_SUCCESS;
    FrameQ_FrameBufInfo       *frameBufInfo;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);
    Assert_isTrue ((frame->numFrameBuffers > frameBufNum),
                    FrameQ_A_invalidParams);

    frameBufInfo = (FrameQ_FrameBufInfo *)&(frame->frameBufInfo[0]);

    if (frameBufInfo[frameBufNum].bufSize <
           (validDataSize + frameBufInfo[frameBufNum].startOffset)) {
        status = FrameQ_E_INVALIDARG;
    }
    else {		    
        frameBufInfo[frameBufNum].validSize = validDataSize;
    }
    return (status);
}

/*
 *  ======== FrameQ_setFrameBufDataStartOffset ========
 *  API to set the valid data start offset of a framebuffer  identified by
 *  frameBufNum in a given frame.
 */
Int32
FrameQ_setFrameBufDataStartOffset (FrameQ_Frame  frame,
                                   UInt32        frameBufNum,
                                   UInt32        dataStartOffset)
{
    Int32                      status       = FrameQ_S_SUCCESS;
    FrameQ_FrameBufInfo        *frameBufInfo;

    Assert_isTrue ((NULL != frame), FrameQ_A_invalidParams);
    Assert_isTrue ((frame->numFrameBuffers > frameBufNum),
                    FrameQ_A_invalidParams);

    frameBufInfo = (FrameQ_FrameBufInfo *)&(frame->frameBufInfo[0]);

    if (frameBufInfo[frameBufNum].bufSize <
            (dataStartOffset + frameBufInfo[frameBufNum].validSize)) {
        status = FrameQ_E_INVALIDARG;
    }
    else {		    
        frameBufInfo[frameBufNum].startOffset = dataStartOffset;
    }
    return (status);
}

/*
 *  ======== FrameQ_isSupportedInterface ========
 *  API to to find out the interface type is supported .
 */
Int32 FrameQ_isSupportedInterface(UInt32 type)
{
    Int32  status = TRUE;

    switch (type)
    {
        case FrameQ_INTERFACE_SHAREDMEM:
            /* Supported interface */
        break;
        default:
            /* unsupported interface type specifiec */
            status = FALSE;
    }

    return (status);
}
/************************* Internal functions *********************************/

/*
 *  ======== FrameQ_Module_startup ========
 */
Int FrameQ_Module_startup(Int phase)
{
    if (FrameQ_module->nameServer != NULL) {
        if (!ti_sdo_utils_NameServer_Module_startupDone()) {
            return (Startup_NOTDONE);
        }
    }
    gFrameQ_nsHandle = FrameQ_module->nameServer;
    return (Startup_DONE);
}

/*
 *  ======== _FrameQ_create ========
 * Internal API.This function calls implementation specific create call and
 * populates the obj with the implementation's handle .It Checks if the name is
 * already exists in the name server if not  puts the  entry in to name server
 * and starts creating it.
 */
Int32
_FrameQ_create (FrameQ_Object  *obj,
                const ti_syslink_ipc_rtos_FrameQ_Params *params,
                Error_Block    *eb)
{
    Int32                  status = FrameQ_S_SUCCESS;
    FrameQ_NameServerEntry entry;
    FrameQ_ShMem_Handle    shMemHandle;
    UInt32                 len;

    if ( (NULL == obj) || ( NULL == params)) {
        status = FrameQ_E_INVALIDARG;
        Error_raise(eb,
                    FrameQ_E_InvalidArg,
                    NULL,
                    0);
    }
    else {
        /* Check if name is already exists in the name server */
        if (FrameQ_module->nameServer != NULL) {
            len = sizeof (FrameQ_NameServerEntry);
            status = NameServer_get ((NameServer_Handle)FrameQ_module->nameServer,
                                     (String)params->commonCreateParams.name,
                                     &entry,
                                     &len,
                                     NULL);
            if (status >= 0) {
                /* Already instance with the name exists */
                status = FrameQ_E_INST_EXISTS;
                Error_raise(eb,
                            FrameQ_E_FrameQCreated,
                            NULL,
                            0);
            }
            else if (status == NameServer_E_FAIL) {
                /* Error happened in NameServer. Pass the error up. */
                status = FrameQ_E_FAIL;
            }
            else {
                /* Go ahead and create as there is no instance with that name
                 * exists.
                 */
                 /* Reinitialize the error block */
                 Error_init(eb);
                 status = FrameQ_S_SUCCESS;
            }
        }
    }

    if (status >= 0) {
        /* Based on the type of interface  mentioned  call create of specific
         * implementation.Currently it has support for FrameQ on shared memory.
         * Need to extend this switch case  to add support for other
         * implementations.
         */
        switch (params->commonCreateParams.ctrlInterfaceType) {
            case FrameQ_INTERFACE_SHAREDMEM:
            {
                /* Create the shared memory based FrameQ  instance.*/
                shMemHandle = FrameQ_ShMem_create(
                               (ti_syslink_ipc_rtos_shMem_FrameQ_ShMem_Params*)
                                                        params->impParams,
                                                        eb);
                if (shMemHandle == NULL) {
                      status = FrameQ_E_FAIL;
/*                    Error_raise(eb,
                                FrameQ_E_FrameQCreateFail,
                                NULL,
                                0);
*/
                }
                else {
                    /* Upcast the handle so that we can call Implementation
                     * specific API using IFrameQ.
                     */
                    obj->handle = FrameQ_ShMem_Handle_upCast(
                                              (FrameQ_ShMem_Handle)shMemHandle);
                    obj->ctrlInterfaceType = FrameQ_INTERFACE_SHAREDMEM;
                    obj->objType       = FrameQ_DYNAMIC_CREATE;
                }
            }
            break;

            default:
                status = FrameQ_E_INVALID_INTERFACE;
                Error_raise(eb,
                            FrameQ_E_InvalidInterfaceType,
                            NULL,
                            0);
        }
    }



    return (status);
}

Int32
 _FrameQ_open (FrameQ_Object* obj,
               const ti_syslink_ipc_rtos_FrameQ_Params* params,
               Error_Block *eb)
{
    Int32    status = FrameQ_S_SUCCESS;

    if ( (NULL == obj) || ( NULL == params)) {
        status = FrameQ_E_INVALIDARG;
        Error_raise(eb,
                    FrameQ_E_InvalidArg,
                    NULL,
                    0);
    }

   return (status);
}


/*TBD The following API definations are temporary  hack.Need to remove*/
xdc_Int32 ti_syslink_ipc_rtos_FrameQ_alloc__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame * frame){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_allocv__F(ti_syslink_ipc_rtos_FrameQ_Object * obj,
ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame * frame, xdc_UInt32 * arg1, xdc_UInt8 * arg2)
{ return -1;}

xdc_Int32
ti_syslink_ipc_rtos_FrameQ_dup__F(ti_syslink_ipc_rtos_FrameQ_Object * obj,
                                 ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame orgframe,
                                 ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame * frame){ return -1;}

xdc_Int32
ti_syslink_ipc_rtos_FrameQ_free__F(ti_syslink_ipc_rtos_FrameQ_Object * obj,
                                   ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame){ return -1;}

xdc_Int32
ti_syslink_ipc_rtos_FrameQ_freev__F(ti_syslink_ipc_rtos_FrameQ_Object * obj,
                                    ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame * frame,
                                    xdc_UInt32 arg1){ return -1;}

xdc_Int32
ti_syslink_ipc_rtos_FrameQ_get__F(ti_syslink_ipc_rtos_FrameQ_Object * obj,
                                  ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame * frame){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_getv__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame * frame, xdc_UInt32 * arg1, xdc_UInt8 * arg2){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_put__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_putv__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame * frame, xdc_UInt32 * arg1, xdc_UInt8 arg2){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_registerNotifier__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, ti_syslink_ipc_rtos_interfaces_IFrameQ_NotifyParams * params){ return -1;}

xdc_UInt32 ti_syslink_ipc_rtos_FrameQ_getFrameBufDataStartOffset__F(ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame, xdc_UInt32 arg1){ return ~(-1);}

xdc_UInt32 ti_syslink_ipc_rtos_FrameQ_getFrameBufSize__F(ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame, xdc_UInt32 arg1){ return (UInt32)-1;}

xdc_UInt32 ti_syslink_ipc_rtos_FrameQ_getFrameBufValidSize__F(ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame, xdc_UInt32 arg1){ return (UInt32)-1;}

xdc_Ptr ti_syslink_ipc_rtos_FrameQ_getFrameBuffer__F(ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame, xdc_UInt32 arg1){ return NULL;}

xdc_UInt32 ti_syslink_ipc_rtos_FrameQ_getNumFrameBuffers__F(ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame){return 0xFFFFFFFF; }

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_setFrameBufDataStartOffset__F(ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame, xdc_UInt32 arg1, xdc_UInt32 arg2){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_setFrameBufValidSize__F(ti_syslink_ipc_rtos_interfaces_IFrameQ_Frame frame, xdc_UInt32 arg1, xdc_UInt32 arg2){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_getNumFrames__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, xdc_UInt32 * arg1){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_getNumFreeFrames__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, xdc_UInt32 * arg1){ return -1;}

xdc_Int32 ti_syslink_ipc_rtos_FrameQ_sendNotify__F(ti_syslink_ipc_rtos_FrameQ_Object * obj, xdc_UInt16 arg1){ return -1;}


xdc_Int32 ti_syslink_ipc_rtos_FrameQ_unregisterNotifier__F(ti_syslink_ipc_rtos_FrameQ_Object * obj){ return -1;}

xdc_Int32
ti_syslink_ipc_rtos_FrameQ_getvNumFreeFrames (ti_syslink_ipc_rtos_FrameQ_Object * obj,
                                              UInt32            numFreeFrames[],
                                              UInt8             freeQId[],
                                              UInt8             numFreeQids)
{
    Int32   status = FrameQ_S_SUCCESS;

    status = FrameQ_getvNumFreeFrames(obj,
                                       numFreeFrames,
                                       freeQId,
                                       numFreeQids);
    return (status);
}

xdc_Int32
ti_syslink_ipc_rtos_FrameQ_getvNumFrames__F (ti_syslink_ipc_rtos_FrameQ_Object * obj,
                       UInt32           numFrames[],
                       UInt8            filledQId[],
                       UInt8            numFilledQids)
{

    return(FrameQ_getvNumFrames(obj,
                                numFrames,
                                filledQId,
                                numFilledQids));
}

/*
 *  ======== FrameQ_isSupportedInterface ========
 *  API to to find out the interface type is supported .
 */
xdc_Int32 ti_syslink_ipc_rtos_FrameQ_isSupportedInterface__F(UInt32 type)
{
    Int32  status = TRUE;

    switch (type)
    {
        case FrameQ_INTERFACE_SHAREDMEM:
            /* Supported interface */
        break;
        default:
            /* unsupported interface type specifiec */
            status = FALSE;
    }

    return (status);
}

