/** 
 *  @file   IClientNotifyMgr.xdc
 *
 *  @brief      Interface file of ClientNotifyMgr.
 *
 *
 *  ============================================================================
 *
 *  Copyright (c) 2008-2009, Texas Instruments Incorporated
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  Contact information for paper mail:
 *  Texas Instruments
 *  Post Office Box 655303
 *  Dallas, Texas 75265
 *  Contact information: 
 *  http://www-k.ext.ti.com/sc/technical-support/product-information-centers.htm?
 *  DCMP=TIHomeTracking&HQS=Other+OT+home_d_contact
 *  ============================================================================
 *  
 */




package ti.syslink.ipc.rtos.interfaces;

import ti.sdo.ipc.GateMP;
import ti.sdo.utils.NameServer;
import ti.sdo.utils.MultiProc;
import ti.sdo.utils.List;
import ti.sdo.ipc.interfaces.INotifyDriver;
import ti.sysbios.knl.Swi;
import ti.sysbios.knl.Task;
import ti.sysbios.knl.Semaphore;

interface IClientNotifyMgr   {

   /* @_nodoc
     *  Delegate writer needs...
     */
    const UInt32 CREATED = 0x05251995;
   /* @_nodoc
    *  Delegate writer needs...
    */
    const UInt32 VERSION = 1;
   /* Type of creation/open */
    const UInt STATIC_CREATE  = 1;
    /*! Type of creation/open */
    const UInt DYNAMIC_CREATE = 2;
    /*! Type of creation/open */
    const UInt STATIC_OPEN    = 4;
    /*! Type of creation/open */
    const UInt DYNAMIC_OPEN   = 8;

    /*!
     *  Maximum length of the  name strings .
     */
    config UInt32  maxNameLen = 32;

    /*!
     *============CACHE_LINESIZE ================
     * Cache line size.
     *
     */
    config  UInt32 CACHE_LINESIZE = 128;

    /*!
     *============ MAX_CLIENTS ================
     * Maximum number of clients supported for instance.
     *
     */
    config UInt32 MAX_CLIENTS = 32;

    /*!
     *============ maxInstances ================
     * Maximum number of instances supported for this module
     *
     */
   config UInt32  maxInstances = 256;

    /*!
     *============ MAX_NOTIFYENTRIES ================
     * Maximum number of notify information entries(callbacks)  that 
     * can registered  for an instance.
     *
     */
    config UInt32 MAX_NOTIFYENTRIES   = 64;
    /*!
     *  ======== CacheFlags ========
     *  Enumeration of ClientNotifyMgr cache flags.
     *
     *  @value (CONTROL_CACHEUSE)  If specified ClientNotifyMgr does cache coherence operations  on it shared control structures.
     *
     */
     enum CacheFlags {
        CONTROL_CACHEUSE = 0x1
    };


    /*!
     *  ======== NotifyType ========
     *  Enumeration of Client notifyMgr notification types..
     *
     *  @value(NOTIFICATION_NONE)   No notification required.
     *
     *  @value(NOTIFICATION_ALWAYS) Notify whenever the other client sends data/frees up space.
     *
     *  @value(NOTIFICATION_ONCE)   Notify when the other side sends data/frees up space. Once the
     *                              notification is done, the notification is disabled until it is
     *                              enabled again.
     *
     *  @value(NOTIFICATION_HDWRFIFO_ALWAYS) Notify whenever the other side sends data/frees up space.
     *                                       This notification is never disabled.
     *
     *  @value(NOTIFICATION_HDWRFIFO_ONCE)   Notify when the other side sends data/frees up space. Once the
     *                                       notification is done, the notification is disabled until it is
     *                                       enabled again. The notification is enabled once the watermark
     *                                       is crossed and does not require buffer to get full/empty..
     */
    enum NotifyType {
        NOTIFICATION_NONE = 0u,
        NOTIFICATION_ALWAYS,
        NOTIFICATION_ONCE,
        NOTIFICATION_HDWRFIFO_ALWAYS,
        NOTIFICATION_HDWRFIFO_ONCE
    };

    /*!
     *  ======== ModuleCfgParams ========
     *  Module wise configuration information.
     *  @field(gate )          Gate for protection.
     *  @field(numProcessors)  Number of other processors in the system.
     *  @field(ProcIds)       Pointer to  array  processorids . This array size should be
     *                         equal to numProcessors.It Denotes the number of processors to
     *                         which Client manager will send notifications
     *
     *  @field(drvHandles)     Array of notify driver handles to the each procId.
     *
     */
    struct Config {
        GateMP.Handle gate;
        UInt32        numProcessors;
    }


    /*!
     *  ======== NotifyDrvHandleInfo ========
     *  NotifyDrvHandleInfo  structure
     *
     *  @field(procId)        Processor identifier.
     *  @field(notifyHandle)  Handle to notify driver created to be used for the processor.
     *
     */
    struct NotifyDrvHandleInfo
    {   UInt16 procId;
        Ptr    notifyHandle;
    }

   /*!
    * ============ FnCbck ================
    *
    *  Signature of the callback function to be registered with the ClientNotifyMgr
    *  component.
    *
    *  @param(handle)
    *              Run-time information provided to the upper layer.
    *  @param(arg)
    *              Fixed argument registered with the Notify component along with
    *              the callback function.
    *  @param(payload)
    *              Payload that need to be passed to the call back function.
    *
    *
    */
    typedef void (*FnCbck) ( Ptr , Ptr , UInt16 );

    /*!
     *============ MAX_SUB_NOTIFYENTRIES ================
     * Maximum number of sub notification entries  for this instance. Each  notfication entry
     * of type NotifyParamInfo will contain this number of sub notification entries for the
     * same call back function. This enables an instance of  this module  to use same call back fxn
     * and watermark conditions for event  notifications to notify the other clients of the instance.
     *
     */
    const UInt32  MAX_SUB_NOTIFYENTRIES = 64;

     const UInt32       TWO =2;
    /*!
     *============ BIT_FLAG_SIZE ================
     * Currently hardcoding. Need to resolve the elem size calcaluation in xdc
     * to get the aligned notifyparam info struct size   compatible with the HLOS side.
     *
     */
    const UInt32  BIT_FLAG_SIZE = 1+ ((MAX_SUB_NOTIFYENTRIES - 1)/32);
 
	const UInt32 SWI_PAYLOAD_ENTRIES = 1024;
	const UInt8  SWI_PRIORITY = 0;


	config UInt32 TSK_PAYLOAD_ENTRIES = 1024;
	config UInt8 TSK_PRIORITY = 2;
	/*
	 *  ======== ClientNotifyMgr_swiFxn ========
	 */
	Void swiFxn(UArg arg0, UArg arg1);
	/*
	 *  ======== ClientNotifyMgr_threadFxn ========
	 */
	Void threadFxn(UArg arg0, UArg arg1);
    /*!
     *============NotifyParamInfo ================
     * Notification paramter info  strcutre. For Each registered client
     * ClientNotifyMgr instance maintains an object of this structure type in
     * the shared memory.
     *
     * @field(clientHandle)   Handle that needs to passed to call back function.
     * @field(waterMarkCond1) Water mark condition 1.
     * @field(waterMarkCond2) Water mark condition 2. Set to -1 if second water mark condition is
     *                        not required.
     * @field(procId)         Processor Id of the client.
     * @field(notifyType)     Notification type.
     * @field(notifyFlag)     Flag indicates whether notification needs to be send or not.
     *                        If TRUE notification will be sent to client.No notification will
     *                        be sent if it false
     * @field(isValid)        Indicated whether this  notification entry is valid or not.
     * @filed(eventFlag)      Flag indicates whether callback function is to be called .
     *                        if TRUE Call back function will be called.
     */
     struct NotifyParamInfo {
        volatile UInt32 clientHandle;
        volatile UInt32 waterMarkCond1;
        volatile UInt32 waterMarkCond2;
        volatile UInt32 bitFlag_notifySubEntries[BIT_FLAG_SIZE];
        volatile UInt16 procId;
        volatile UInt16 notifyType;
        volatile UInt16 isValid;
        volatile UInt8  waitOnMultiNotifyFlags;
        volatile UInt8  notifyFlag[MAX_SUB_NOTIFYENTRIES];
        volatile UInt8  eventFlag[MAX_SUB_NOTIFYENTRIES];
        volatile UInt8  forceNotify;
     }

    /*!
     *============ClientNotifyMgr_registerNoitifyParams ================
     * Structure defining  client notify information.
     * ClientNotifyMgr_registerClient AI takes this as argument during
     * call back function registration.
     *
     * @field(clientHandle)  Client instance handle which is going to be registered.
     * @field(procId)          Processor Id of the client..
     * @field(notifyType)    Notification type.
     * @field(fxnPtr)          Call back function
     * @filed(cbContext)    Call back context pointer.Will be passed as an arg to call
     *                              back function.
     * @field(waterMarkCond1)   water mark condition 1.
     * @field(waterMarkCond2)   water mark condition 2. Set to -1 if second water mark condition is
     *                          not required.
     */
    struct registerNoitifyParams {
            Ptr          clientHandle;
            UInt16       procId;
            UInt32       notifyType;
            FnCbck       fxnPtr;
            Ptr          cbContext;
            UInt32       watermarkCond1;
            UInt32       watermarkCond2;
    };

    /*! @nodoc
      *============ClientNotifyMgr_eventListenerInfo ================
      * ClientNotifyMgr_eventListenerInfo structure.
      *
      * @field(listElem)  List element.
      * @field(cbckFxn)   Call back function.
      * @field(notifyId)  Notification id of this client.
      * @field(cbContext) Conetxt pointer.
      */

    struct eventListenerInfo
    {
        List.Elem    listElem;
        FnCbck       cbckFxn;
        UInt32       notifyId;
        Ptr          cbContext;
    };

    /*! @nodoc
      *============callbackCtx ================
      * ClientNotifyMgr_eventListenerInfo structure.
      *
      * @field(gate)      gate provided for the instance.
      * @field(control)   Shared control information of the instance.
      * @field(notifyId)  Cache usage flag.
      */
    struct callbackCtx {
        GateMP.Handle                  gate;
        volatile Attrs               *attrs;
        volatile Ptr                  notifyInfoObjBaseAddr;
        UInt32                        minAlign;
        Bool                          cacheEnabled;
		UInt32                        instId;
    };

    /*! @nodoc
      *============ swiPostInfo ================
      * Structue used by ClientNotifyMgr_cbFxn to keep track of how many times 
	  * ClientNotifyMgr_cbFxn  posts the SWI and to keep track of valid notifyPayloadInfo entries 
	  * in the module that  SWI function has to process.
      *
      */
   struct swiPostInfo {
        Swi.Handle						swiHandle;
		UInt32                          notifyCnt;
		UInt32                          readIndex;
		UInt32                          writeIndex;
    };

    /*! @nodoc
      *============ TaskPostInfo ================
      * Structue used by ClientNotifyMgr_thFxn to keep track of how many times 
	  * ClientNotifyMgr_thFxn  posts the Semaphore and to keep track of valid notifyPayloadInfo entries 
	  * in the module that  SWI function has to process.
      *
      */
   struct TaskPostInfo {
        Task.Handle                     tskHandle;
        Semaphore.Handle				semHandle;
		UInt32                          notifyCnt;
		UInt32                          readIndex;
		UInt32                          writeIndex;
    };
	  /*! @nodoc
      *============ notifyPayloadInfo ================
      * notifyPayloadInfo structure used by ClientNotifyMgr_cbFxn to push 
	  * notifypayload and callback context to swis.
      *
      * @field(gate)      gate provided for the instance.
      * @field(control)   Shared control information of the instance.
      * @field(notifyId)  Cache usage flag.
	  * @field(notifyCnt) Counter to keep track of the number of notify isrs executed for this instance to call SWI/Semaphore_posts s in case 
	  *                   of  instance local protection is GateMP_LOCALPROTECT_TASKLET/GateMP_LOCALPROTECT_THREAD/GateMP_LOCALPROTECT_PROCESS.
      */

    struct notifyPayloadInfo {
        Ptr                             cliCallbackCtx;
		UInt32                          notifyPayload;
    };

     
     /*!
     *  ======== Attrs ========
     *  Shared  control strucrue for the instance attributes.
     *
     *  @field(status)         flag  denoting whether instance is created or not.
     *  @field(version)        Version of the  instance.
     *  @filed(creatorProcId)  processor id of the crator
     *  @field(eventNo)        EventNymebr used for notifying the clients of this instance.
     *
     *  @field(maxClients)     Max number of clients allowed for this instance..
     *  @field(numNotifyEntries)Number of notifyinfo Objects required to be maintained by this instance.
     *                          This should be equal to the number of call backs  registered By the  user of this instance. 
     *  @field(numSubNotifyEntries)  Denotes the number of sub entries present
     *                         in each NotifyParamInfo filed of the instannce.
     *                         Enables client of the instance to use same notify
     *                         info( such as water mark) to check different
     *                         conditions in the other clients. This allows
     *                         FrameQBufMgr to use same call back function to
     *                         handle  notifications  for individual FramePools
     *                         separately by having registered with
     *                         clientNotifyMgr once and Passing subNotifyEntryId
     *                         ( equalvalentto FramePoolNo) as additional arg to
     *                         enable/disable/sendNotification multi functions.
     *  @field(numActClients)  Number of clients currently using this instance.
     *
     */
    struct Attrs {
        volatile UInt32 status;
        volatile UInt32 version;
        volatile UInt32 creatorProcId;
        volatile UInt32 eventNo;
        volatile UInt32 maxClients; /* Maximum number of clients  that can
                                     * use this instance
                                     */
        volatile UInt32 numNotifyEntries;
        volatile UInt32 numSubNotifyEntries;
        volatile UInt32 numActClients; /* Number of clients currently
                                        * using this instance
                                        */
        volatile UInt16 entryId;
    }

   /*!
     *  ======== getConfig ========
     * API to return the module wide configurable parameters.
     *
     *  @param(cfgParams)   Pointer to parameter structure to be initialized.
     */
    Int32 getConfig (Config * cfgParams);


    /*!
     *  ======== setNameServer ========
     *  Fnction to create a nameServer.
     *
     *  @param(name)       Name of the name server instance that needs to be created.
     *
     *
     *  @param(gate)      gate for critical regioons in name server instance.
     *
     *  @b(returns)       returns handle.
     *
     */
     Ptr setNameServer( String name , GateMP.Handle gate);

    /*!
     *  ======== useNameServer ========
     *  Whether to have this module use the NameServer or not.
     *
     *  Currently defaulted to false because static creation of NameServer
     *  instances is not supported.
     */
    metaonly config Bool useNameServer = true;

    /*!
     *  ======== nameSrvPrms ========
     *  This Params object is used for temporary storage of the
     *  module wide parameters that are for setting the NameServer instance.
     */
    metaonly config NameServer.Params nameSrvPrms;

    /*!
     *  ======== tableSection ========
     *  Section name is used to place the names table
     */
    metaonly config String tableSection = null;

    /*!
     *============nameServer ================
     * Handle to the name server.
     *
     */
     config NameServer.Handle  nameServer = null;

    /*!
     *  ======== sharedMemModReq ========
     *  Amount of shared memory required for initializing the module.
     *
     *  Can be used to make sure the shared buffer is large
     *  enough before calling initialize.
     *
     *  All parameters except sharedAddr and sharedAddrSize must be
     *  the same that will be used in the {@link #initialize} function.
     *
     *
     *  @b(returns)          Amount of shared memory required for module initialization.
     */
    UInt32 sharedMemModReq ();

     instance:

     /*!
     *  ======== name ========
     *  Name for the instance .
     *
     */
     config String name = null;
    /*!
     *  ======== eventNo ========
     *  EventNumber to be used for notification .
     *
     */
     config UInt32 eventNo = 0;
    /*!
     *  ======== openFlag ========
     *  Boolean to indicate whether it is  actual create call or open call .if set to FALSE,instance create call
     *  Opens the already created instance.
     *
     */
     config Bool openFlag = false;
    /*!
     *============creatorProcId ================
     * Processor id of the caller  instane create caller. Delete will be allowed only by the creator.
     *
     */
    config UInt32 creatorProcId = -1;


    /*!
     *============sharedMemAddr ================
     *
     * Shared memory base addres for the ClientNotifyMgr instance.
     *
     */
    config  Ptr sharedAddr = null;
    /*!
     *============ sharedMemSize ================
     *
     * Shared memory size for the ClientNotifyMgr instance.
     *
     */
     config  UInt32 sharedAddrSize = 0;

    /*!
     *  GateMP used for critical region management of the shared memory
     *
     *  This must be a shared memory gate.
     */
    config GateMP.Handle gate = null;

    /*!
     * Cache flags denoting whether cache coherence operations are  needed to be performed on instance shared control structures.
     */
    config   UInt32 cacheFlags = 0;

    /*!
     * Number of notifyinfo Objects required to be maintained by this instance.This should be equal to the number of call backs  registered
     * By the  user of this instance.
     */

    config UInt32  numNotifyEntries = 1; 
    /*!
     * Sub notifiy entries in each notifyParamInfo entry.
     */
    config UInt32 numSubNotifyEntries = 1;

    /*! 
     *  ======== regionId ========
     *  SharedRegion ID.
     *
     *  The ID corresponding to the index of the shared region in which this
     *  shared instance is to be placed.  This is used in create() only when
     *  {@link #name} is not null.
     */
    config UInt16 regionId = 0;

 /*!
     *  ======== registerClient ========
     *  Function to register notification with the ClientNotifyMgr instance.
     *
     *  @param(params) client notify  info struct
     *  @param(id)     Pointer to hold the identifier returned after successfully returning from the registerClient API.
     *
     *  @b(returns)    SUCCESS if successfully registered the client notification information with the ClientNotifyMgr.
     *
     */
     Int32 registerClient(registerNoitifyParams *params,
                          UInt32 *Id);
   /*!
     *  ======== registerClient ========
     *  Function to unregister notification with the ClientNotifyMgr instance.
     *
     *  @param(id)         Identifier obtained through {@link #registerClient} call.
     *
     *  @b(returns)        SUCCESS if successfully registered the client notification information with the ClientNotifyMgr.
     *
     */
     Int32 unregisterClient( UInt32 Id);

    /*!
     *  ======== enableNotification ========
     *  Function to enable notification for the caller client.
     *
     *  @param(Id)         Id obtained through {@link #registerClient} call.
     *  @param(subId)      Sub notification entry in which to enable notifications. {@link #Attrs.numSubNotifyEntries} call.
     *
     *  @b(returns)        SUCCESS if successfully enabled the  notification for the caller client in the ClientNotifyMgr
     *                     instance.
     *
     */
     Int32 enableNotification (UInt32 Id,UInt32 subId );

    /*!
     *  ======== waitOnMultiSubEntryNotify ========
     *  Function to enable notification for the caller client.
     *
     *  @param(Id)         Id obtained through {@link #registerClient} call.
     *  @param(subId)      Sub notification entrys on which client wants to set
     *                     notify flag to true and wants  only one call back for
     *                     all the the sub notification entries.
     *                    {@link #Attrs.numSubNotifyEntries} call.
     *  @param(numSubEntries) Number of subnotification entries.
     *
     *  @b(returns)        SUCCESS if successfully enabled the  notification for
     *                     the caller client in the ClientNotifyMgr instance.
     *
     */
     Int32 waitOnMultiSubEntryNotify(UInt32 Id,
                                     UInt32 subId[],
                                     UInt32 numSubEntries);

    /*!
     *  ======== disableNotification ========
     *  Function to disable notification for the caller client.
     *
     *  @param(Id)         Id obtained through {@link #registerClient} call.
     *
     *
     *  @b(returns)        SUCCESS if successfully disabled the  notification for the caller client in the ClientNotifyMgr
     *                     instance.
     *
     */
     Int32 disableNotification (UInt32 Id,UInt32 subId );


    /*!
     *  ======== sendNotification ========
     *  Function to send notification to the clients of ClientNotifyMgr instance.
     *
     *  @param(Id)         Id obtained through {@link #registerClient} call.

     *  @param(arg1)       Value to checked against water mark. For FrameQBufMgr it should
     *                     be number of free buffers whose size  is  arg2 value.
     *  @param(arg2)       Size of the buffers.
     *
     *
     *  @b(returns)        SUCCESS if successfully sendthe  notification.
     *
     */
     Int32 sendNotificationMulti (UInt32 Id,UInt32 subId, UInt32 arg1 , UInt32 arg2 );

    /*!
     *  ======== sendNotification ========
     *  Function to send notification to the clients of ClientNotifyMgr instance.
     *
     *  @param(Id)         Id of the client to which notification needs to be sent.
     *                     see {@link #registerClient}
     *  @param(arg1)       Value to checked against water mark. For FrameQBufMgr it should
     *                     be number of free buffers whose size  is  arg2 value.
     *  @param(arg2)       Size of the buffers.
     *
     *
     *  @b(returns)        SUCCESS if successfully sendthe  notification.
     *
     */
     Int32 sendNotification (UInt32 Id,UInt32 subId, UInt32 arg1 , UInt32 arg2 );

    Int32
    sendForceNotification (UInt32                  id,
                           UInt16                  payLoad);
    Int32
    sendForceNotificationMulti (UInt32                  Id,
                                UInt16                  payLoad);

    /*!
     *  ======== close ========
     *  Function to close the dynamically opened instance using {@link #open} call.
     *
     *
     *  @b(returns)     returns 0 if  successfully registered the call back function.
     *
     */
    Int32 close();


}
